clear all;
close all;
% This is a Script!
% 
% Name:     script_ebrf1_R1.m
% 
% Version:  v0.1
% 
% Purpose:  This sript is intended to perform the generalized schur
%           decomposition and plot the result in the parameter space, see
%           Gasteiger (2014, JMCB).
% 
% Author:   Emanuel Gasteiger
%           
% Date:     13.07.2010                  Last Revision:     16.05.2014
% 
% 
% Model:    See Gasteiger (2014, JMCB).
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (A) Model set-up:
%     We take the system as in McCallum(2009,p.16ff.) equations
% 
%     (18) y(t) = A   * E[y(t+1)|omega(t)] + CE  * y(t-1) + D   * u(t)
% 
%          mx1  = mxm * mx1                + mxm * mx1    + mxn * nx1 
% 
%     (19) E[u(t+1)|omega(t)] =   P   * u(t)
% 
%                       n x 1 = n x n * n x 1
% 
%     where we can rewrite (18) as a system of first-order:
%              _                 _             _        _
%             | E[y(t+1)|omega(t)]|           |   y(t)   |
%     Abar  * |                   | = CEbar * |          | + Dbar * u(t).
%             |_      y(t)       _|           |_ y(t-1) _|
% 
%     2mx2m *       2mx1            = 2mx2m *     2mx1     + 2mxn * nx1
% 
%     with m predetermined and n exogeneous variables, see
%     McCallum(2009,p.15).
% 
%     We assume AR(1) shock processes of the form
%     E[u(t+1)|omega(t)] = P * u(t)
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (B) Calibration:
% Expectations parameter
alpha   = 0:0.1:1;
alpha   = sort(alpha',1,'ascend');
theta    = 0.10:0.01:2.00;
theta    = sort(theta',1,'ascend');

% Structural parameters
sigma   =   0.157;
beta    =   0.99;
lambda  =   0.024;
omegax   =   0.01:0.01:2.00;
omegax   =   omegax';

% Number of variables
n=2;
 
% % Pre-allocating memory for vectors and matrices
GEVresult = zeros(n*2, max(size(omegax)), size(alpha,1), size(theta,1));
 
tic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (C) Loop:
for kk=1:max(size(theta))
    for k=1:max(size(alpha))
        for i=1:max(size(omegax))
        
%--------------------------------------------------------------------------
% (i) Matrices
Abar = [
        0,      -(alpha(k,1)*beta*lambda)/(lambda^2 + omegax(i,1)), 0, 0;
        0,  (alpha(k,1)*beta*omegax(i,1))/(lambda^2 + omegax(i,1)), 0, 0;
        0,                                        0,                1, 0;
        0,                                        0,                0, 1 ...
       ];

CEbar = [
    1, 0,           -omegax(i,1)/(lambda^2 + omegax(i,1)), -(beta*lambda*theta(kk,1)^2*(alpha(k,1) - 1))/(lambda^2 + omegax(i,1));
    0, 1,  -(lambda*omegax(i,1))/(lambda^2 + omegax(i,1)),  (beta*omegax(i,1)*theta(kk,1)^2*(alpha(k,1) - 1))/(lambda^2 + omegax(i,1));
    1, 0,                                               0,	0;
    0, 1,                                               0,	0 ... 
        ];

%--------------------------------------------------------------------------
% (ii) Generalized Schur Decomposition of the matrix pencil
%     [CEbar - lambda * Abar]
  [T,S,Z,Q] = qz(CEbar',Abar'); % NOTE: % Using the Transpose of CEbar and
                                        % Abar yields upper triangular
                                        % matrix T. Matrix S would be upper
                                        % triangular anyway.    
    DS=diag(S);
    DT=diag(T);
    GEV = abs(DT./DS);
    GEV = sort(GEV);
    
    % We build the matrix of generalized eigenvalues
    GEVresult(:,i,k,kk)=GEV;
	
        end
    end
end
save('./EBRF1/GEVresult_ebrf1_R1','GEVresult');
toc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (D) Plots:
%--------------------------------------------------------------------------
% (i) Omega-theta region plot for various alpha
szalpha1={	'\alpha = 0.0';'\alpha = 0.1';'\alpha = 0.2';'\alpha = 0.3';...
			'\alpha = 0.4';'\alpha = 0.5';'\alpha = 0.6';'\alpha = 0.7';...
			'\alpha = 0.8';'\alpha = 0.9';'\alpha = 1.0'};
colormap(gray)
szalpha3={	'otrchart_ebrf1_true_alpha00';'otrchart_ebrf1_true_alpha01';...
			'otrchart_ebrf1_true_alpha02';'otrchart_ebrf1_true_alpha03';...
			'otrchart_ebrf1_true_alpha04';'otrchart_ebrf1_true_alpha05';...
			'otrchart_ebrf1_true_alpha06';'otrchart_ebrf1_true_alpha07';...
			'otrchart_ebrf1_true_alpha08';'otrchart_ebrf1_true_alpha09';...
			'otrchart_ebrf1_true_alpha10'};

for k=1:max(size(alpha))
    test2 = squeeze(GEVresult(:,:,k,:));
    test2 = permute(test2,[3 2 1]);
    test2 = floor(test2(:,:,:));
    figure
%         spy(test2(:,:,4),'b',12) ;hold all;
        spy(test2(:,:,3),'g',12) ;hold all;
        spy(test2(:,:,2),'y',12) ;hold all;
%         spy(test2(:,:,1),'y',12) ;hold all;
        xlim([0 200]);
        set(gca,'FontSize',12);
        set(gca,'XTick',0:10:200);
        set(gca,'XTickLabel',{  '0';'';'';'';'';'0.5';'';'';'';'';'1.0';
                                    '';'';'';'';'1.5';'';'';'';'';'2.0';});
        set(gca,'YTick',0:10:200);
        set(gca,'YTickLabel',{  '0.1';'';'';'';'0.5';'';'';'';'';'1.0';
                                    '';'';'';'';'1.5';'';'';'';'';'2.0';});
%         set(gca,'YTick',0:50:200);
%         set(gca,'YTickLabel',{  '0.9';'';'1.0';'';'1.1';});
        set(ylabel('\theta','Interpreter','tex','FontUnits','points',...
                   'FontSize',24));
        set(xlabel('\omega_x','Interpreter','tex','FontUnits','points',...
                   'FontSize',24));
        set(legend('Determinate','Explosive',2),'Interpreter','tex',...
                   'Location','NorthEast','Box', 'on','Color', 'white',...
                   'FontSize',8,'Position',[0.630 0.733 0.2 0.2]);
        title(szalpha1{k},'Interpreter','tex','FontUnits','points',...
              'FontSize',24);
        az = 0;el = 270;view(az, el);
	saveas(gcf,['./EBRF1/' szalpha3{k}], 'png');
end