clear all;
close all;
% This is a Script!
% 
% Name:     script_ebrf2_R2_theta0110.m
% 
% Version:  v0.1
% 
% Purpose:  This sript is intended to perform the generalized schur
%           decomposition and plot the result in the parameter space, see
%           Gasteiger (2014, JMCB).
% 
% Author:   Emanuel Gasteiger
%           
% Date:     13.07.2010                  Last Revision:     16.05.2014
% 
% 
% Model:    See Gasteiger (2014, JMCB).
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (A) Model set-up:
%     We take the system as in McCallum(2009,p.16ff.) equations
% 
%     (18) y(t) = A   * E[y(t+1)|omega(t)] + CE  * y(t-1) + D   * u(t)
% 
%          mx1  = mxm * mx1                + mxm * mx1    + mxn * nx1 
% 
%     (19) E[u(t+1)|omega(t)] =   P   * u(t)
% 
%                       n x 1 = n x n * n x 1
% 
%     where we can rewrite (18) as a system of first-order:
%              _                 _             _        _
%             | E[y(t+1)|omega(t)]|           |   y(t)   |
%     Abar  * |                   | = CEbar * |          | + Dbar * u(t).
%             |_      y(t)       _|           |_ y(t-1) _|
% 
%     2mx2m *       2mx1            = 2mx2m *     2mx1     + 2mxn * nx1
% 
%     with m predetermined and n exogeneous variables, see
%     McCallum(2009,p.15).
% 
%     We assume AR(1) shock processes of the form
%     E[u(t+1)|omega(t)] = P * u(t)
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (B) Calibration:
% Expectations parameter
alpha   = 0:0.1:1;
alpha   = sort(alpha',1,'ascend');
theta   = 1.10; 	% where theta in {0.90,1.00,1.05,1.10}

% Structural parameters
sigma   =   0.157;
beta    =   0.99;
lambda  =   0.024;
omegax   =   0.000001:0.01:2.00;
omegax   =   omegax';
omegai   =   0.000001:0.01:2.00;
omegai   =   omegai';

% Number of variables
n=5;
 
% % Pre-allocating memory for vectors and matrices
GEVresult = zeros(n*2, max(size(omegai)), max(size(omegax)), size(alpha,1));

% Switch off warning
s = warning('off','MATLAB:divideByZero');

tic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (C) Loop:
    for k=1:max(size(alpha))
        for j=1:max(size(omegai));
                for i=1:max(size(omegax))

%--------------------------------------------------------------------------
% (i) Matrices
% Coefficients from implicit instrument rule
gamma1	=	omegax(i,1)/(sigma*omegai(j,1)); 
gamma2  =   lambda/(sigma*omegai(j,1));
gamma3  =   1 + (lambda/beta*sigma) + (1/beta);
gamma4  =   (1/beta);
gamma0  =   (1 + 2*alpha(k,1) * (1-alpha(k,1) * theta^2 * beta));
gamma5  =	beta^2 * (1-alpha(k,1)) * theta^2 * gamma1;
gamma6  =   (1-alpha(k,1))* theta^2 * (beta + (lambda/beta*sigma) + beta^2);
gamma7  =   (1-alpha(k,1))^2 * theta^4 * beta^3;

% Coefficients from expectations-based reaction function based on
% implicit instrument rule
mupi	=   ( (1/gamma0)*(gamma1+gamma2*lambda)*(1/sigma) ...
                + (gamma2/gamma0)*beta ) * alpha(k,1) ...
            / (1+(1/gamma0)*(gamma1+gamma2*lambda)*(1/sigma));
mux     =   ( (1/gamma0)*(gamma1+gamma2*lambda) - (gamma5/gamma0)  ) ...
            / (1+(1/gamma0)*(gamma1+gamma2*lambda)*(1/sigma));
muL0	=   ( (1/gamma0)*(gamma1+gamma2*lambda) * ((1-alpha(k,1))*theta^2)...
                - (gamma1/gamma0)) ...
            / (1+(1/gamma0)*(gamma1+gamma2*lambda)*(1/sigma));
muL1	=   gamma3 * alpha(k,1)...
            / (1+(1/gamma0)*(gamma1+gamma2*lambda)*(1/sigma));
muL2	=   -gamma4 * alpha(k,1)^2 ...
            / (1+(1/gamma0)*(gamma1+gamma2*lambda)*(1/sigma));
muL3    =   ( (1/gamma0)*(gamma1+gamma2*lambda)*(1/sigma) ...
                + (gamma2/gamma0)* beta ) ...
            * (1-alpha(k,1)) * theta^2 ...
            / (1+(1/gamma0)*(gamma1+gamma2*lambda)*(1/sigma));
mui1    =   gamma6 / (1+(1/gamma0)*(gamma1+gamma2*lambda)*(1/sigma));
mui2    =   - gamma7 / (1+(1/gamma0)*(gamma1+gamma2*lambda)*(1/sigma));

Abar = [
	0,                                               0,                                               0,                                               1,                                               0,                                               0,                                               0,                                               0,                                               0,                                               0;
	-1/sigma*mui2,                               alpha(k,1)-1/sigma*mux,                      1/sigma*alpha(k,1)-1/sigma*mupi,                                   -1/sigma*mui1,                                               0,                                               0,                                               0,                                               0,                                               0,                                               0;
	-lambda/sigma*mui2,                   lambda*alpha(k,1)-lambda/sigma*mux, lambda/sigma*alpha(k,1)+alpha(k,1)*beta-lambda/sigma*mupi,                              -lambda/sigma*mui1,                                               0,                                               0,                                               0,                                               0,                                               0,                                               0;
	mui2,	mux,	mupi,	mui1,	0,	0,	0,	0,	0,	0;
	0,	0,	0,	0,	0,	0,	0,	0,	0,	0;
	0,	0,	0,	0,	0,	1,	0,	0,	0,	0;
	0,	0,	0,	0,	0,	0,	1,	0,	0,	0;
	0,	0,	0,	0,	0,	0,	0,	1,	0,	0;
	0,	0,	0,	0,	0,	0,	0,	0,	1,	0;
	0,	0,	0,	0,	0,	0,	0,	0,	0,	1;
        ];

CEbar = [
	1,	0,	0,	0,	0,	0,	0,	0,	0,	0;
	0,	1,	0,	0,	0,	0,	-(1-alpha(k,1))*theta^2+1/sigma*muL0,                                  -1/sigma*(1-alpha(k,1))*theta^2+1/sigma*muL3,                                                             1/sigma*muL1,                                                             1/sigma*muL2;
	0,	0,	1,	0,	0,	0,	-lambda*(1-alpha(k,1))*theta^2+lambda/sigma*muL0, -lambda/sigma*(1-alpha(k,1))*theta^2-beta*(1-alpha(k,1))*theta^2+lambda/sigma*muL3,                                                        lambda/sigma*muL1,                                                        lambda/sigma*muL2;
	0,	0,	0,	1,	0,	0,	-muL0,	-muL3,	-muL1,	-muL2;
	0,	0,	0,	0,	1,	0,	0,	0,	-1,	0;
	1,	0,	0,	0,	0,	0,	0,	0,	0,	0;
	0,	1,	0,	0,	0,	0,	0,	0,	0,	0;
	0,  0,	1,	0,	0,	0,	0,	0,	0,	0;
	0,	0,	0,	1,	0,	0,	0,	0,	0,	0;
	0,	0,	0,	0,	1,	0,	0,	0,	0,	0;
        ];

%--------------------------------------------------------------------------
% (ii) Generalized Schur Decomposition of the matrix pencil
%     [CEbar - lambda * Abar]
  [T,S,Z,Q] = qz(CEbar',Abar'); % NOTE: % Using the Transpose of CEbar and
                                        % Abar yields upper triangular
                                        % matrix T. Matrix S would be upper
                                        % triangular anyway.    
    DS=diag(S);
    DT=diag(T);
    GEV = abs(DT./DS);
    GEV = sort(GEV);
    
    % We build the matrix of generalized eigenvalues
    GEVresult(:,j,i,k)=GEV;
            end
        end
    end
save('./EBRF2/GEVresult_ebrf2_R2_theta0110','GEVresult');
toc

% Restore warning
warning(s);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (D) Plots:
%--------------------------------------------------------------------------
% (i) Omegax-omegai region plot for various alpha
szalpha1={	'\alpha = 0.0 | \theta = 1.10';'\alpha = 0.1 | \theta = 1.10';...
            '\alpha = 0.2 | \theta = 1.10';'\alpha = 0.3 | \theta = 1.10';...
			'\alpha = 0.4 | \theta = 1.10';'\alpha = 0.5 | \theta = 1.10';...
            '\alpha = 0.6 | \theta = 1.10';'\alpha = 0.7 | \theta = 1.10';...
			'\alpha = 0.8 | \theta = 1.10';'\alpha = 0.9 | \theta = 1.10';...
            '\alpha = 1.0 | \theta = 1.10'};
szalpha3={	'oorchart_ebrf2_true_alphatheta011000';...
            'oorchart_ebrf2_true_alphatheta011001';...
			'oorchart_ebrf2_true_alphatheta011002';...
            'oorchart_ebrf2_true_alphatheta011003';...
			'oorchart_ebrf2_true_alphatheta011004';...
            'oorchart_ebrf2_true_alphatheta011005';...
			'oorchart_ebrf2_true_alphatheta011006';...
            'oorchart_ebrf2_true_alphatheta011007';...
			'oorchart_ebrf2_true_alphatheta011008';...
            'oorchart_ebrf2_true_alphatheta011009';...
			'oorchart_ebrf2_true_alphatheta011010'};

for k=1:max(size(alpha))
    test2 = squeeze(GEVresult(:,:,:,k));
    test2 = permute(test2,[2 3 1]);
    test2 = floor(test2(:,:,:));
    figure
%         spy(test2(:,:,10),'b',12) ;hold all;
%         spy(test2(:,:,9),'b',12) ;hold all;
%         spy(test2(:,:,8),'b',12) ;hold all;
%         spy(test2(:,:,7),'b',12) ;hold all;
        spy(test2(:,:,6),'g',12) ;hold all;
        spy(test2(:,:,5),'y',12) ;hold all;
        spy(test2(:,:,4),'y',12) ;hold all;
        spy(test2(:,:,3),'y',12) ;hold all;
        spy(test2(:,:,2),'y',12) ;hold all;
        spy(test2(:,:,1),'y',12) ;hold all;
        xlim([0 200]);
        set(gca,'FontSize',12);
        set(gca,'XTick',0:10:200);
        set(gca,'XTickLabel',{  '0';'';'';'';'';'0.5';'';'';'';'';'1.0';
                                    '';'';'';'';'1.5';'';'';'';'';'2.0';});
        set(gca,'YTick',0:10:200);
        set(gca,'YTickLabel',{  '0';'';'';'';'';'0.5';'';'';'';'';'1.0';
                                    '';'';'';'';'1.5';'';'';'';'';'2.0';});
        set(ylabel('\omega_i','Interpreter','tex','FontUnits','points',...
                   'FontSize',24));
        set(xlabel('\omega_x','Interpreter','tex','FontUnits','points',...
                   'FontSize',24));
        set(legend('Determinate','Explosive',2),'Interpreter','tex',...
                   'Location','NorthWest','Box', 'on','Color', 'white',...
                   'FontSize',8,'Position',[0.630 0.733 0.2 0.2]);
        title(szalpha1{k},'Interpreter','tex','FontUnits','points',...
              'FontSize',24);
        az = 0;el = 270;view(az, el);
  	saveas(gcf,['./EBRF2/' szalpha3{k}], 'png');
end