clear all;
close all;
% This is a Script!
% 
% Name:     script_oir_theta09.m
% 
% Version:  v0.1
% 
% Purpose:  This sript is intended to perform the generalized schur
%           decomposition and plot the result in the parameter space, see
%           Gasteiger (2014, JMCB).
% 
% Author:   Emanuel Gasteiger
%           
% Date:     13.07.2010                  Last Revision:     16.05.2014
% 
% 
% Model:    See Gasteiger (2014, JMCB).
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (A) Model set-up:
%     We take the system as in McCallum(2009,p.16ff.) equations
% 
%     (18) y(t) = A   * E[y(t+1)|omega(t)] + CE  * y(t-1) + D   * u(t)
% 
%          mx1  = mxm * mx1                + mxm * mx1    + mxn * nx1 
% 
%     (19) E[u(t+1)|omega(t)] =   P   * u(t)
% 
%                       n x 1 = n x n * n x 1
% 
%     where we can rewrite (18) as a system of first-order:
%              _                 _             _        _
%             | E[y(t+1)|omega(t)]|           |   y(t)   |
%     Abar  * |                   | = CEbar * |          | + Dbar * u(t).
%             |_      y(t)       _|           |_ y(t-1) _|
% 
%     2mx2m *       2mx1            = 2mx2m *     2mx1     + 2mxn * nx1
% 
%     with m predetermined and n exogeneous variables, see
%     McCallum(2009,p.15).
% 
%     We assume AR(1) shock processes of the form
%     E[u(t+1)|omega(t)] = P * u(t)
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (B) Calibration:
% Expectations parameter
alpha   = 0:0.1:1;
alpha   = sort(alpha',1,'ascend');
theta   = 0.90; 	% where theta in {0.90,1.00,1.10}

% Structural parameters
sigma   =   0.157;
beta    =   0.99;
lambda  =   0.024;
omegax   =   0.000001:0.01:2.00;
omegax   =   omegax';
omegai   =   0.000001:0.01:2.00;
omegai   =   omegai';

% Number of variables
n=5;
 
% % Pre-allocating memory for vectors and matrices
GEVresult = zeros(n*2, max(size(omegai)), max(size(omegax)), size(alpha,1));

% Switch off warning
s = warning('off','MATLAB:divideByZero');

tic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (C) Loop:
    for k=1:max(size(alpha))
        for j=1:max(size(omegai));
                for i=1:max(size(omegax))

%--------------------------------------------------------------------------
% (i) Matrices
gamma1	=	omegax(i,1)/(sigma*omegai(j,1)); 
gamma2  =   lambda/(sigma*omegai(j,1));
gamma3  =   1 + (lambda/beta*sigma) + (1/beta);
gamma4  =   (1/beta);
gamma0  =   (1 + 2*alpha(k,1) * (1-alpha(k,1) * theta^2 * beta));
gamma5  =	beta^2 * (1-alpha(k,1)) * theta^2 * gamma1;
gamma6  =   (1-alpha(k,1))* theta^2 * (beta + (lambda/beta*sigma) + beta^2);
gamma7  =   (1-alpha(k,1))^2 * theta^4 * beta^3;

Abar = [
0,0,0,1,0,0,0,0,0,0;
1/(gamma0*sigma+lambda*gamma2+gamma1)*gamma7,                        1/(gamma0*sigma+lambda*gamma2+gamma1)*gamma0*sigma*alpha(k,1)+1/(gamma0*sigma+lambda*gamma2+gamma1)*gamma5,                     1/(gamma0*sigma+lambda*gamma2+gamma1)*gamma0*alpha(k,1)-gamma2/(gamma0*sigma+lambda*gamma2+gamma1)*alpha(k,1)*beta,                                                                                -1/(gamma0*sigma+lambda*gamma2+gamma1)*gamma6,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0;
lambda/(gamma0*sigma+lambda*gamma2+gamma1)*gamma7,              lambda/(gamma0*sigma+lambda*gamma2+gamma1)*gamma0*sigma*alpha(k,1)+lambda/(gamma0*sigma+lambda*gamma2+gamma1)*gamma5, lambda/(gamma0*sigma+lambda*gamma2+gamma1)*gamma0*alpha(k,1)+(gamma0*sigma+gamma1)/(gamma0*sigma+lambda*gamma2+gamma1)*alpha(k,1)*beta,                                                                           -lambda/(gamma0*sigma+lambda*gamma2+gamma1)*gamma6,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0;
-1/(gamma0*sigma+lambda*gamma2+gamma1)*sigma*gamma7,    (lambda*gamma2+gamma1)/(gamma0*sigma+lambda*gamma2+gamma1)*sigma*alpha(k,1)-1/(gamma0*sigma+lambda*gamma2+gamma1)*sigma*gamma5, (lambda*gamma2+gamma1)/(gamma0*sigma+lambda*gamma2+gamma1)*alpha(k,1)+gamma2/(gamma0*sigma+lambda*gamma2+gamma1)*sigma*alpha(k,1)*beta,                                                                           1/(gamma0*sigma+lambda*gamma2+gamma1)*sigma*gamma6,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0,                                                                                                                            0;
0,0,0,0,0,0,0,0,0,0;
0,0,0,0,0,1,0,0,0,0;
0,0,0,0,0,0,1,0,0,0;
0,0,0,0,0,0,0,1,0,0;
0,0,0,0,0,0,0,0,1,0;
0,0,0,0,0,0,0,0,0,1;
       ];

CEbar = [
1,0,0,0,0,0,0,0,0,0;
0,1,0,0,0,0,-1/(gamma0*sigma+lambda*gamma2+gamma1)*gamma0*sigma*(1-alpha(k,1))*theta^2-1/(gamma0*sigma+lambda*gamma2+gamma1)*gamma1*alpha(k,1),                     -1/(gamma0*sigma+lambda*gamma2+gamma1)*gamma0*(1-alpha(k,1))*theta^2+gamma2/(gamma0*sigma+lambda*gamma2+gamma1)*beta*(1-alpha(k,1))*theta^2,                                                                                                    1/(gamma0*sigma+lambda*gamma2+gamma1)*gamma3*alpha(k,1),                                                                                                 -1/(gamma0*sigma+lambda*gamma2+gamma1)*gamma4*alpha(k,1)^2;
0,0,1,0,0,0,-lambda/(gamma0*sigma+lambda*gamma2+gamma1)*gamma0*sigma*(1-alpha(k,1))*theta^2-lambda/(gamma0*sigma+lambda*gamma2+gamma1)*gamma1*alpha(k,1), -lambda/(gamma0*sigma+lambda*gamma2+gamma1)*gamma0*(1-alpha(k,1))*theta^2-(gamma0*sigma+gamma1)/(gamma0*sigma+lambda*gamma2+gamma1)*beta*(1-alpha(k,1))*theta^2,                                                                                               lambda/(gamma0*sigma+lambda*gamma2+gamma1)*gamma3*alpha(k,1),                                                                                            -lambda/(gamma0*sigma+lambda*gamma2+gamma1)*gamma4*alpha(k,1)^2;
0,0,0,1,0,0,-(lambda*gamma2+gamma1)/(gamma0*sigma+lambda*gamma2+gamma1)*sigma*(1-alpha(k,1))*theta^2+1/(gamma0*sigma+lambda*gamma2+gamma1)*sigma*gamma1*alpha(k,1), -(lambda*gamma2+gamma1)/(gamma0*sigma+lambda*gamma2+gamma1)*(1-alpha(k,1))*theta^2-gamma2/(gamma0*sigma+lambda*gamma2+gamma1)*sigma*beta*(1-alpha(k,1))*theta^2,                                                                                             -1/(gamma0*sigma+lambda*gamma2+gamma1)*sigma*gamma3*alpha(k,1),                                                                                            1/(gamma0*sigma+lambda*gamma2+gamma1)*sigma*gamma4*alpha(k,1)^2;
0,0,0,0,1,0,0,0,-1,0;
1,0,0,0,0,0,0,0,0,0;
0,1,0,0,0,0,0,0,0,0;
0,0,1,0,0,0,0,0,0,0;
0,0,0,1,0,0,0,0,0,0;
0,0,0,0,1,0,0,0,0,0;
        ];

%--------------------------------------------------------------------------
% (ii) Generalized Schur Decomposition of the matrix pencil
%     [CEbar - lambda * Abar]
  [T,S,Z,Q] = qz(CEbar',Abar'); % NOTE: % Using the Transpose of CEbar and
                                        % Abar yields upper triangular
                                        % matrix T. Matrix S would be upper
                                        % triangular anyway.    
    DS=diag(S);
    DT=diag(T);
    GEV = abs(DT./DS);
    GEV = sort(GEV);
    
    % We build the matrix of generalized eigenvalues
    GEVresult(:,j,i,k)=GEV;
            end
        end
    end
save('./OIR/GEVresult_oir_theta09','GEVresult');
toc

% Restore warning
warning(s);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (D) Plots:
%--------------------------------------------------------------------------
% (i) Omegax-omegai region plot for various alpha
szalpha1={	'\alpha = 0.0 | \theta = 0.9';'\alpha = 0.1 | \theta = 0.9';...
            '\alpha = 0.2 | \theta = 0.9';'\alpha = 0.3 | \theta = 0.9';...
			'\alpha = 0.4 | \theta = 0.9';'\alpha = 0.5 | \theta = 0.9';...
            '\alpha = 0.6 | \theta = 0.9';'\alpha = 0.7 | \theta = 0.9';...
			'\alpha = 0.8 | \theta = 0.9';'\alpha = 0.9 | \theta = 0.9';...
            '\alpha = 1.0'};
szalpha3={	'oorchart_oir_alphatheta0900';'oorchart_oir_alphatheta0901';...
			'oorchart_oir_alphatheta0902';'oorchart_oir_alphatheta0903';...
			'oorchart_oir_alphatheta0904';'oorchart_oir_alphatheta0905';...
			'oorchart_oir_alphatheta0906';'oorchart_oir_alphatheta0907';...
			'oorchart_oir_alphatheta0908';'oorchart_oir_alphatheta0909';...
			'oorchart_oir_alphatheta0910'};

for k=1:max(size(alpha))
    test2 = squeeze(GEVresult(:,:,:,k));
    test2 = permute(test2,[2 3 1]);
    test2 = floor(test2(:,:,:));
    figure
%         spy(test2(:,:,10),'b',12) ;hold all;
%         spy(test2(:,:,9),'b',12) ;hold all;
%         spy(test2(:,:,8),'b',12) ;hold all;
        spy(test2(:,:,7),'b',12) ;hold all;
        spy(test2(:,:,6),'g',12) ;hold all;
        spy(test2(:,:,5),'y',12) ;hold all;
        spy(test2(:,:,4),'y',12) ;hold all;
        spy(test2(:,:,3),'y',12) ;hold all;
        spy(test2(:,:,2),'y',12) ;hold all;
        spy(test2(:,:,1),'y',12) ;hold all;
        xlim([0 200]);
        set(gca,'FontSize',12);
        set(gca,'XTick',0:10:200);
        set(gca,'XTickLabel',{  '0';'';'';'';'';'0.5';'';'';'';'';'1.0';
                                    '';'';'';'';'1.5';'';'';'';'';'2.0';});
        set(gca,'YTick',0:10:200);
        set(gca,'YTickLabel',{  '0';'';'';'';'';'0.5';'';'';'';'';'1.0';
                                    '';'';'';'';'1.5';'';'';'';'';'2.0';});
        set(ylabel('\omega_i','Interpreter','tex','FontUnits','points',...
                   'FontSize',24));
        set(xlabel('\omega_x','Interpreter','tex','FontUnits','points',...
                   'FontSize',24));
        set(legend('Indeterminate','Determinate','Explosive',3),'Interpreter','tex',...
                   'Location','NorthWest','Box', 'on','Color', 'white',...
                   'FontSize',8,'Position',[0.630 0.733 0.2 0.2]);
        title(szalpha1{k},'Interpreter','tex','FontUnits','points',...
              'FontSize',24);
        az = 0;el = 270;view(az, el);
	saveas(gcf,['./OIR/' szalpha3{k}], 'png');
end